@@ -143,7 +143,8 @@ def check(recipe_path, no_download_source=False, config=None, variants=None, **k
 
 
 def build(recipe_paths_or_metadata, post=None, need_source_download=True,
-          build_only=False, notest=False, config=None, variants=None, **kwargs):
+          build_only=False, notest=False, config=None, variants=None, stats=None,
+          **kwargs):
     """Run the build step.
 
     If recipe paths are provided, renders recipe before building.
@@ -159,6 +160,11 @@ def build(recipe_paths_or_metadata, post=None, need_source_download=True,
 
     config = get_or_merge_config(config, **kwargs)
 
+    # if people don't pass in an object to capture stats in, they won't get them returned.
+    #     We'll still track them, though.
+    if not stats:
+        stats = {}
+
     recipe_paths_or_metadata = _ensure_list(recipe_paths_or_metadata)
     for recipe in recipe_paths_or_metadata:
         if not any((hasattr(recipe, "config"), isinstance(recipe, string_types))):
@@ -189,11 +195,11 @@ def build(recipe_paths_or_metadata, post=None, need_source_download=True,
 
     if not absolute_recipes:
         raise ValueError('No valid recipes found for input: {}'.format(recipe_paths_or_metadata))
-    return build_tree(absolute_recipes, build_only=build_only, post=post, notest=notest,
-                      need_source_download=need_source_download, config=config, variants=variants)
+    return build_tree(absolute_recipes, config, stats, build_only=build_only, post=post,
+                      notest=notest, need_source_download=need_source_download, variants=variants)
 
 
-def test(recipedir_or_package_or_metadata, move_broken=True, config=None, **kwargs):
+def test(recipedir_or_package_or_metadata, move_broken=True, config=None, stats=None, **kwargs):
     """Run tests on either packages (.tar.bz2 or extracted) or recipe folders
 
     For a recipe folder, it renders the recipe enough to know what package to download, and obtains
@@ -205,12 +211,18 @@ def test(recipedir_or_package_or_metadata, move_broken=True, config=None, **kwar
     else:
         config = get_or_merge_config(config, **kwargs)
 
+    # if people don't pass in an object to capture stats in, they won't get them returned.
+    #     We'll still track them, though.
+    if not stats:
+        stats = {}
+
     with config:
         # This will create a new local build folder if and only if config
         #   doesn't already have one. What this means is that if we're
         #   running a test immediately after build, we use the one that the
         #   build already provided
-        test_result = test(recipedir_or_package_or_metadata, config=config, move_broken=move_broken)
+        test_result = test(recipedir_or_package_or_metadata, config=config, move_broken=move_broken,
+                           stats=stats)
     return test_result
 
 
