@@ -357,6 +357,11 @@ default_structs = {
     'build/ignore_run_exports': list,
     'build/requires_features': dict,
     'build/provides_features': dict,
+    'build/pre-link': text_type,
+    'build/post-link': text_type,
+    'build/pre-unlink': text_type,
+    'build/string': text_type,
+    'build/pin_depends': text_type,
     'requirements/build': list,
     'requirements/host': list,
     'requirements/run': list,
@@ -368,8 +373,6 @@ default_structs = {
     'test/commands': list,
     'test/imports': list,
     'package/version': text_type,
-    'build/string': text_type,
-    'build/pin_depends': text_type,
     'source/svn_rev': text_type,
     'source/git_tag': text_type,
     'source/git_branch': text_type,
@@ -501,7 +504,8 @@ def build_string_from_metadata(metadata):
         #    nice to get rid of this, since the hash supercedes that functionally, but not clear
         #    whether anyone's tools depend on this file naming right now.
         for s, names, places in (('np', 'numpy', 2), ('py', 'python', 2), ('pl', 'perl', 2),
-                                 ('lua', 'lua', 2), ('r', ('r', 'r-base'), 3)):
+                                 ('lua', 'lua', 2), ('r', ('r', 'r-base'), 3),
+                                 ('mro', 'mro-base', 3)):
             for ms in metadata.ms_depends('run'):
                 for name in ensure_list(names):
                     if ms.name == name and name in build_pkg_names:
